@@ -5,7 +5,7 @@ use strict;
 use Class::Autouse qw{Carp Locale::Language Locale::Object::Country Locale::Object::DB I18N::AcceptLanguage I18N::LangTags::Detect Storable};
 
 use vars qw($VERSION);
-our $VERSION = '0.54';
+our $VERSION = '0.55';
 
 =head1 NAME
 
@@ -13,7 +13,7 @@ CGI::Lingua - Create a multilingual web page
 
 =head1 VERSION
 
-Version 0.54
+Version 0.55
 
 =cut
 
@@ -570,8 +570,13 @@ sub _find_language {
 				return;
 			}
 		}
-		unless($self->{_rlanguage}) {
-			$self->{_rlanguage} = 'Unknown';
+		if(((!$self->{_rlanguage}) || ($self->{_rlanguage} eq 'Unknown')) &&
+		   ((length($http_accept_language) == 2) || ($http_accept_language =~ /^..-..$/))) {
+			$self->{_rlanguage} = $self->_code2language($http_accept_language);
+
+			unless($self->{_rlanguage}) {
+				$self->{_rlanguage} = 'Unknown';
+			}
 		}
 		$self->{_slanguage} = 'Unknown';
 	}
@@ -750,22 +755,7 @@ sub country {
 
 	my $ip = $ENV{'REMOTE_ADDR'};
 
-	unless($ip) {
-		return();
-	}
-	if($self->{_cache}) {
-		$self->{_country} = $self->{_cache}->get($ip);
-		if($self->{_logger}) {
-			if(defined($self->{_country})) {
-				$self->{_logger}->debug("Get $ip from cache = $self->{_country}");
-			} else {
-				$self->{_logger}->debug("$ip isn't in the cache");
-			}
-		}
-		if(defined($self->{_country})) {
-			return $self->{_country};
-		}
-	}
+	return unless(defined($ip));
 
 	require Data::Validate::IP;
 	Data::Validate::IP->import();
@@ -788,6 +778,20 @@ sub country {
 		return;
 	}
 
+	if($self->{_cache}) {
+		$self->{_country} = $self->{_cache}->get($ip);
+		if($self->{_logger}) {
+			if(defined($self->{_country})) {
+				$self->{_logger}->debug("Get $ip from cache = $self->{_country}");
+			} else {
+				$self->{_logger}->debug("$ip isn't in the cache");
+			}
+		}
+		if(defined($self->{_country})) {
+			return $self->{_country};
+		}
+	}
+
 	if($self->{_have_ipcountry} == -1) {
 		if(eval { require IP::Country; }) {
 			IP::Country->import();
